import translation
from optparse import OptionParser

DEFAULT_PARSER = OptionParser()

def get_default_option_parser():
  parser = DEFAULT_PARSER
  parser.add_option("-V", "--verify",
                    action="store_true", dest="verify", default=False,
                    help="Verify host configuration tasks")
  parser.add_option("-C", "--configure",
                    action="store_true", dest="configure", default=False,
                    help="Configure this host for attachment")
  parser.add_option("-R", "--rescan",
                    action="store_true", dest="rescan", default=False,
                    help="Rescan devices")
  return parser

def get_fc_option_parser():
  parser = get_default_option_parser()
  parser.add_option("-P", "--print",
                    action="store_true", dest="print_storage_ports", default=False,
                    help="Print WWPN of HBA devices")
  return parser

def get_iscsi_option_parser():
  parser = get_default_option_parser()
  parser.add_option("-P", "--print",
                    action="store_true", dest="print_storage_ports", default=False,
                    help="Print iSCSI initiator name and alias")
  parser.add_option("-c", "--connect",
                    action="store", dest="connect", help="Discover new iSCSI target")
  parser.add_option("-d", "--disconnect",
                    action="store", dest="disconnect", help="Disconnetct from an iSCSI target")
  return parser


def default_handler(attachment, options):
  if options.verify:
    for task in get_config_tasklist(attachment):
      print task
  if options.configure:
    attachment.config()
  if options.print_storage_ports:
    for port in attachment.get_storage_ports():
      print str(port)
  if options.rescan:
    attachment.rescan()

def fc_handler(attachment, options):
  default_handler(attachment, options)

def iscsi_handler(attachment, options):
  default_handler(attachment, options)
  if options.connect:
    attachment.connect_iscsi(options.connect)
  if options.disconnect:
    attachment.disconnect_iscsi(options.connect)

def get_config_tasklist(attachment):
    tasks = attachment.config_tasks()
    if not tasks:
      return [translation.HOST_CONFIGURED]
    return tasks

