#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import re
from commands import getstatusoutput

class ExternalCommand(object):
    def __init__(self, command=''):
        self.command = command

    def execute(self):
        command_string = self.command
        logging.debug('executing: ' + command_string)
        statusoutput = getstatusoutput(command_string)
        self.return_value = statusoutput[0]
        logging.debug('execution returned: ' + str(self.return_value))
        self.output = statusoutput[1]
        logging.debug('execution output: ' + self.output)
        if (self.return_value == 32512):
            raise IOError(32512, self.output)
        if (self.return_value == 32256):
            raise IOError(32256, self.output)
        return True

    def grep(self, string=''):
        lines = []
        for line in self.output.splitlines():
            if line.__contains__(string):
                lines.append(line)
        return lines

    def contains(self, string =''):
        for line in self.output.splitlines():
            if line.__contains__(string):
                return True
        return False

    def head(self, number=10):
        all_lines = self.output.splitlines()
        head_lines = []
        for i in range(0,number):
            if (i<all_lines.__len__()):
                head_lines.append(all_lines[i])
        return head_lines

    def tail(self, number=10):
        all_lines = self.output.splitlines()
        tail_lines = []
        first_line_number = all_lines.__len__() - number
        if (first_line_number < 0):
            first_line_number = 0
        for i in range (first_line_number, all_lines.__len__()):
            tail_lines.append(all_lines[i])
        return tail_lines

    def search_and_replace(self, pattern_string, replace_string):
        return re.sub(pattern_string, replace_string, self.output)

    def is_error_free(self):
        for line in self.output.splitlines():
            if line.__contains__('command not found'):
                 return False
            elif line.__contains__('No such file'):
                return False
            elif line.__contains__('error') or line.__contains__('failed'):
               return False
            elif line.__contains__('ERROR') or line.__contains__('FAILED'):
               return False
        return True

    def print_output(self):
        for line in self.output.splitlines():
            print line.strip()
