#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging

from execute import ExternalCommand
from scsi_host import ScsiHost
from qlogic_host import QlogicHost
from emulex_host import EmulexHost
from lsi_host import LsiHost

class FiberChannelHosts(object):
    def __init__(self):
#        self.log_used_classes()
        self.scsi_hosts = []
        self.build_scsi_hosts()

    def log_used_classes(self):
        self.logger.log_a_class(QlogicHost)
        self.logger.log_a_class(EmulexHost)
        self.logger.log_a_class(LsiHost)
        self.logger.log_a_class(ScsiHost)

    def get_qlogic_fc_hosts(self):
        hosts_lists = self.get_fc_hosts_for_a_specific_driver('qla')
        for fc_host in hosts_lists:
            self.scsi_hosts.append(QlogicHost(fc_host))
        return hosts_lists
    
    def get_emulex_fc_hosts(self):
        hosts_lists = self.get_fc_hosts_for_a_specific_driver('lpfc')
        for fc_host in hosts_lists:
            self.scsi_hosts.append(EmulexHost(fc_host))
        return hosts_lists

    def get_lsi_fc_hosts(self):
        hosts_lists = self.get_fc_hosts_for_a_specific_driver('mptfc')
        for fc_host in hosts_lists:
            self.scsi_hosts.append(LsiHost(fc_host))
        return hosts_lists

    def get_fc_hosts_for_a_specific_driver(self, driver_name):
        scsi_hosts = ExternalCommand('ls -d /sys/bus/pci/drivers/' + driver_name + '*/*/*host* 2> /dev/null')
        scsi_hosts.execute()
        return scsi_hosts.search_and_replace('.*host', 'host').splitlines()

    def build_scsi_hosts(self):
        all_fc_hosts_list = self.get_fc_hosts_information_from_sysfs()
        qlogic_fc_hosts_list = self.get_qlogic_fc_hosts()
        emulex_fc_hosts_list = self.get_emulex_fc_hosts()
        lsi_fc_hosts_list    = self.get_lsi_fc_hosts()
        for fc_host in all_fc_hosts_list:
            supported = False
            if qlogic_fc_hosts_list.__contains__(fc_host):
                supported = True
            if emulex_fc_hosts_list.__contains__(fc_host):
                supported = True
            if    lsi_fc_hosts_list.__contains__(fc_host):
                supported = True
            if not supported:
                    logging.debug('Unsupported Fibre-Channel HBA found: %s' % (fc_host))
                    self.scsi_hosts.append(ScsiHost(fc_host))
    
    def get_fc_hosts_information_from_sysfs(self):
        scsi_hosts = ExternalCommand('ls -d /sys/class/fc_host/* 2> /dev/null')
        scsi_hosts.execute()
        return scsi_hosts.search_and_replace('.*host','host').splitlines()

    def scan_all_hosts(self):
        for scsi_host in self.scsi_hosts:
            scsi_host.scan()
        self.refresh_multipath_maps()

    def refresh_multipath_maps(self):
        multipath_flush = ExternalCommand('multipath -F')
        multipath_flush.execute()
        multipath_run = ExternalCommand('multipath')
        multipath_run.execute()
    
    def get_scsi_hosts(self):
        return self.scsi_hosts

    def print_wwn(self):
        for scsi_host in self.scsi_hosts:
            logging.info('WWN of %s (%s): %s' % (scsi_host.host_name, scsi_host.type, scsi_host.wwn))

    def modprobe_sg(self):
        modprobe_sg_command = ExternalCommand('''modprobe sg''')
        modprobe_sg_command.execute()

