#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging

class File(object):
    def __init__(self, path):
        self.filepath = path

    def identical_to_file(self, dest_path):
        try:
            dest_data = self.read_file(dest_path)
        except IOError:
            raise IOError
        try:
            source_data = self.read_file()
        except IOError:
            raise IOError
        if source_data == dest_data:
            logging.debug('%s and %s are identical' % (self.filepath, \
                                                           dest_path))
            return True
        logging.debug('%s and %s are not identical' % (self.filepath, \
                                                           dest_path))
        return False

    def copy_to(self, dest_path):
        try:
            source_data = self.read_file()
        except IOError:
            raise IOError
        return self.write_file(dest_path, source_data)

    def file_exists(self):
        try:
            file_object = open(self.filepath, 'r')
            file_object.close()
        except IOError:
            logging.debug('could not open/close the file %s' % \
                              self.filepath)
            return False
        return True

    def read_file(self, filepath=''):
        if filepath == '':
            filepath = self.filepath
        try:
            file_object = open(filepath, 'r')
            data = file_object.read()
            file_object.close()
        except IOError:
            logging.debug('could not read the file %s' % filepath)
            raise IOError
        return data

    def write_file(self, filename='', data='', mode='w+'):
        return self.write_lines_to_file(filename, data.splitlines(), mode)

    def write_lines_to_file(self, filepath='', lines=[], mode='w+'):
        if filepath == '':
            filepath = self.filepath
        try:
            file_object = open(filepath, mode)
            for line in lines:
                file_object.write('%s\n' % line)
            file_object.flush()
            file_object.close()
        except IOError:
            logging.error('could not write file %s' % filepath)
            return False
        return True
