#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------


import logging, definitions

class Logger(object):
    def __init__(self, prefix='host_attach'):
        self.indent_level = 0;
        self.prefix = prefix
        self.logfile_path = definitions.LOGFILE_PATH
        self.initialize_python_logging_module()

    def initialize_python_logging_module(self):
        self.console_logger = logging.Logger('host_attach')
        self.logfile_logger = logging.Logger('host_attach')
        self.initialize_python_logging_to_console()
        try:
            self.initialize_python_logging_to_logfile()
        except IOError:
            self.console_logger.warn('Cannot write to logfile %s' \
                                     % (self.logfile_path))
            pass

    def initialize_python_logging_to_console(self):
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)
        console_formatter = logging.Formatter('%(levelname)s: %(message)s')
        console_handler.formatter = console_formatter
        self.console_logger.addHandler(console_handler)

    def initialize_python_logging_to_logfile(self):
        logfile_handler = logging.FileHandler(self.logfile_path, 'a')
        logfile_handler.setLevel(logging.DEBUG)
        logfile_formatter = logging.Formatter(datefmt='%Y-%m-%d %H:%M:%S', \
                                              fmt='%(asctime)s '\
                                              '%(levelname)-8s %(message)s')
        logfile_handler.formatter = logfile_formatter
        self.logfile_logger.addHandler(logfile_handler)

    def entry(self, logged_class, function, *args, **kwargs):
        arguments_formatted = ", ".join(str(arg) for arg in args)
        keyword_arguments_formatted = ", ".join("%s=%s" % \
                                                (key, value) for key, value \
                                                in kwargs.items())
        function_formatted = '%s.%s' % (logged_class.__name__, function.__name__)
        entry_message_formatted = '> %s (%s, %s)' % (function_formatted, \
                                                     arguments_formatted, \
                                                     keyword_arguments_formatted)
        self.debug(entry_message_formatted)
        self.indent_level += 1

    def exit(self, logged_class, function, result):
        self.indent_level -= 1
        exit_message_formatted = '< %s.%s = %s' % (logged_class.__name__, \
                                                   function.__name__, result)
        self.debug(exit_message_formatted)

    def debug(self, message):
        self.log(logging.DEBUG, message)

    def info(self, message):
        self.log(logging.INFO, message)

    def warn(self, message):
        self.log(logging.WARN, message)

    def error(self, message):
        self.log(logging.ERROR, message)

    def log(self, level, message):
        self.console_logger.log(level, message)
        identation_formatted = '%s: %s' % (self.prefix, ' '*self.indent_level)
        if message[0] != '>' and message[0] != '<':
            message = ' ' + message
        if self.logfile_logger.handlers.__len__() > 0:
            self.logfile_logger.log(level, '%s %s' %(identation_formatted, \
                                                     message))

    def log_a_class(self, class_to_trace):
        def logger(func):
            def callable(*args, **kwargs):
                self.entry(class_to_trace, func, *args, **kwargs)
                result = func(*args, **kwargs)
                self.exit(class_to_trace, func, result)
                return result
            callable.__name__ = func.__name__
            return callable
        for attribute_name, attribute_value in class_to_trace.__dict__.iteritems():
            if type(attribute_value).__name__ == 'function':
                setattr(class_to_trace, attribute_name, logger(attribute_value))
