#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import os

from hostattachment.exceptions import HostAttachmentError

class Iscsi(object):
    def __init__(self, backup_path):
#        self.logger.log_a_class(IscsiDetector)
        iscsi_cls = Iscsi.detect_iscsi_method()
        self.iscsi_object = iscsi_cls(backup_path)

    @staticmethod
    def detect_iscsi_method():
        """Decides if it's OpenIscsi (modern) or LinuxIscsi (deprecated), and
        returns the right object"""
        if os.path.exists("/sbin/iscsiadm"):
            logging.debug("OpenIscsi detected")
            from open_iscsi import OpenIscsi
            return OpenIscsi
        elif os.path.exists("/sbin/iscsi-device"):
            logging.debug("LinuxIscsi detected")
            from linux_iscsi import LinuxIscsi
            return LinuxIscsi
        else:
            raise HostAttachmentError("No iSCSI component detected")

    def is_configured(self):
        if self.get_configuration_task_list():
            return False
        else:
            return True

    def get_configuration_task_list(self):
        return self.iscsi_object.get_configuration_task_list()

    def configure(self):
        return self.iscsi_object.configure()

    def connect(self, ip_address):
        if not self.is_configured():
            raise HostAttachmentError("iSCSI is not configured")
        else:
            return self.iscsi_object.connect(ip_address)


    def disconnect(self, ip_address):
        if not self.is_configured():
            raise HostAttachmentError("iSCSI is not configured")
        else:
            return self.iscsi_object.disconnect(ip_address)

    def get_iqn(self):
        return self.iscsi_object.get_iqn()

    def print_all(self):
        self.iscsi_object.print_all()

    def scan(self):
        self.iscsi_object.scan()

