#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging

import ip_address as IpAddress
from iscsid_conf import ConfigFileLinuxIscsi

from execute import ExternalCommand

class ConnectionManager(object):
    def __init__(self, backup_path):
        self.iscsi_conf = ConfigFileLinuxIscsi('/etc/iscsi.conf', backup_path)
    
    def connect(self, ip_address):
        if not IpAddress.is_alive(ip_address):
            logging.error('IP address %s is not responding, aborting' % ip_address)
            return False
        logging.info('adding IP address %s to the configuration' % ip_address)
        self.discover(ip_address)
        return True

    def disconnect(self, ip_address):
        logging.info('removing IP address %s from the configuration' % ip_address)
        self.undiscover(ip_address)
        return True

    def verify_discovered_address(self, ip_address):
        for address in self.iscsi_conf.discovery_addresses:
            if address == ip_address:
                return True
        return False

    def discover(self, ip_address):
        self.iscsi_conf.add_discovery_address(ip_address)
        self.iscsi_conf.write_to_file()

    def undiscover(self, ip_address):
        self.iscsi_conf.remove_discovery_address(ip_address)
        self.iscsi_conf.write_to_file()

    def iscsi_scan(self):
        start = ExternalCommand('/etc/init.d/iscsi restart')
        start.execute()
        scan = ExternalCommand('iscsi-rescan')
        scan.execute()

    def is_configured(self):
        return True

    def print_all(self):
        self.print_discoveries()
        self.print_connections()

    def print_connections(self):
        logging.info('iSCSI Connections')
        print_command = ExternalCommand('iscsi-ls')
        print_command.execute()
        for line in print_command.output.splitlines():
            print line

    def print_discoveries(self):
        logging.info('iSCSI Discovery Addresses')
        for address in self.iscsi_conf.discovery_addresses:
            print 'DiscoveryAddress=%s' % address

