#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import definitions

from iscsi.classes import ConfigFileReader, ConfigFileWriter, IIscsidConf, \
    IConfigFile

class ConfigFileLinuxIscsi(IConfigFile):
    def __init__(self, filepath, backup_path):
        self.discovery_addresses = []
        self.filepath = filepath
        self.backup_path = backup_path
        self.settings = {}
        self.reader = ConfigFileReader(self.filepath)
        self.read()
    
    def analyze(self):
        for line in self.reader.config_lines:
            (name, value) = self.split_n_strip_line(line)
            logging.debug('found %s=%s' % (name, value))
            if name == 'DiscoveryAddress':
                self.add_discovery_address(value)
            else:
                self.add_setting(name, value)

    def add_discovery_address(self, value):
        if not self.discovery_addresses.__contains__(value):
            self.discovery_addresses.append(value)

    def remove_discovery_address(self, value):
        if self.discovery_addresses.__contains__(value):
            self.discovery_addresses.remove(value)
    
    def write_to_file(self, filepath=''):
        if filepath == '':
            filepath = self.filepath 
        writer = ConfigFileWriter(filepath, self.settings, self.discovery_addresses)
        return writer.write_to_file()

    def is_empty(self):
        if len(self.settings) == 0 and len(self.discovery_addresses) == 0:
            return True
        return False
        
class IscsidConfLinuxIscsi(IIscsidConf):
    def __init__(self, backup_path):
        self.backup_path = backup_path
        self.source_file = definitions.ISCSI_CONF_SOURCE
        self.set_dest_filename()
        self.load_config()

    def load_config(self):
        self.source_config = ConfigFileLinuxIscsi(\
            self.source_file, self.backup_path)
        self.dest_config = ConfigFileLinuxIscsi(\
            self.dest_filename, self.backup_path)
        self.source_config.read()
        self.dest_config.read()

    def set_dest_filename(self):
        self.dest_filename = '/etc/iscsi.conf'


