#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging

import ip_address as IpAddress
from execute import ExternalCommand
from nextra import Nextra

class ConnectionManager(object):
    def __init__(self, backup_path):
#        self.log_used_classes()
        pass

    def log_used_classes(self):
        self.logger.log_a_class(Nextra)

    def connect(self, ip_address):
        if not IpAddress.is_alive(ip_address):
            logging.error('IP address %s is not responding' % ip_address)
            return False
        nextra = Nextra(ip_address)
        if not nextra.query():
            logging.error('failed to query XIV array on IP Address %s' % ip_address)
            return False
        nextra.discover_from_all_portals()
        nextra.connect_to_all_portals()
        return True
        
    def disconnect(self, ip_address):
        nextra = Nextra(ip_address)
        nextra.disconnect_from_portal(ip_address)
#       The stuff below doesn't work if we can't query the portal (i.e. it's
#       down)
#        nextra.query()
#        nextra.disconnect_from_all_portals()
#        nextra.undiscover_from_all_portals()

    def is_configured(self):
        return True

    def print_all(self):
        self.print_discoveries()
        self.print_nodes()
        self.print_sessions()
        print ''

    def print_discoveries(self):
        print 'discovered portals and targets:'
        discovery = ExternalCommand('iscsiadm -m discovery')
        discovery.execute()
        discovery.print_output()
        print ''

    def print_nodes(self):
        print 'discovered portals and targets:'
        nodes = ExternalCommand('iscsiadm -m node')
        nodes.execute()
        nodes.print_output()
        print ''

    def print_sessions(self):
        print 'current connected portals and targets:'
        sessions = ExternalCommand('iscsiadm -m session')
        sessions.execute()
        sessions.print_output()
        print ''

    def refresh_connections(self):
        logging.info('refreshing iSCSI connections')
        command = ExternalCommand('iscsiadm -m node -U all')
        command.execute()
        command = ExternalCommand('iscsiadm -m node -L all')
        command.execute()

