#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import os
import definitions

from iscsi.classes import ConfigFileReader, ConfigFileWriter, IIscsidConf, \
    IConfigFile

class ConfigFileOpenIscsi(IConfigFile):
    def __init__(self, filepath, backup_path):
        self.filepath = filepath
        self.backup_path = backup_path
        self.settings = {}
        self.reader = ConfigFileReader(self.filepath)
        self.read()

class IscsidConfOpenIscsi(IIscsidConf):
    conf_filename = 'iscsid.conf'
    def __init__(self, backup_path):
        self.backup_path = backup_path
        self.source_file = definitions.ISCSID_CONF_SOURCE
        self.set_dest_filename()
        self.load_config()

    def load_config(self):
        self.source_config = ConfigFileOpenIscsi(\
            self.source_file, self.backup_path)
        self.dest_config = ConfigFileOpenIscsi(\
            self.dest_filename, self.backup_path)
        self.source_config.read()
        self.dest_config.read()

    def set_dest_filename(self):
        if self.is_conf_in_dir("/etc/iscsi"):
            self.dest_filename = '/etc/iscsi/%s' % self.conf_filename
        elif self.is_conf_in_dir("/etc"):
            self.dest_filename = '/etc/%s' % self.conf_filename
        else: 
            os.mkdir("/etc/iscsi")
            os.symlink("/etc/%s" % (self.conf_filename), \
                "/etc/iscsi/%s" % (self.conf_filename))
            self.dest_filename = '/etc/%s' % self.conf_filename

    def is_conf_in_dir(self, dir_path):
        return os.path.exists("%s%s%s" % (dir_path, os.path.sep, self.conf_filename))


