#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging

from os import path
from execute import ExternalCommand

MODULES_DIR = path.join('/','lib','modules')

class KernelModule(object):

    def __init__(self, module_name):
        self.module_name = module_name
        self.module_path = None
        self.get_kernel_parameters()
        self.get_module_path()

    def get_kernel_parameters(self):
        self.get_kernel_release()

    def get_kernel_release(self):
        uname_command = ExternalCommand('uname -r')
        uname_command.execute()
        self.kernel_release = uname_command.head()[0]
        
    
    def get_module_path(self):
        find_command = ExternalCommand('find %s -name %s.ko' % (MODULES_DIR, self.module_name))
        find_command.execute()
        if not find_command.is_error_free():
            logging.debug('did not find kernel module %s.ko' % self.module_name)
            self.module_path = None
        else:
            self.module_path = find_command.grep('%s/' % self.kernel_release)[0]

    def is_module_alive(self):
        if not self.module_path:
            logging.debug('module %s does not exist' % self.module_name)
            return False
        lsmod_command = ExternalCommand('lsmod')
        lsmod_command.execute()
        if not lsmod_command.is_error_free():
            return False
        if lsmod_command.contains(self.module_name):
            return True
        return False
    
    def insert_module(self):
        if self.is_module_alive():
            logging.debug('module is already alive')
            return True
        if not self.module_path:
            logging.debug('module %s does not exist' % self.module_name)
            return False
        insmod_command = ExternalCommand('insmod %s' % self.module_path)
        insmod_command.execute()
        if not insmod_command.is_error_free():
            return False
        return True

