#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import definitions
from file import File
from device import Device

class ConfigFileReader(object):
    def __init__(self, filepath=definitions.MULTIPATH_CONF_SOURCE):
        self.filepath = filepath
        self.config_string = ''
        self.config_lines = []

    def read(self):
        config_file = File(self.filepath)
        try:
            self.config_string = config_file.read_file()
        except IOError:
            logging.debug('failed to read %s' % self.filepath)
            return False
        self.strip_comments_and_newlines_and_tabs()
        return True

    def strip_comments_and_newlines_and_tabs(self):
        for line in self.config_string.splitlines():
            stripped_line = line.strip().replace('\t', ' ')
            if not stripped_line == '' and not stripped_line.startswith('#'):
                logging.debug('found a non-empty an not-commented line: %s' % stripped_line)
                self.config_lines.append(stripped_line)

    def get_block(self, block_type):
        (block_start, block_end) = self.get_block_line_numbers(block_type)
        if block_start>=0 and block_end>=0 and block_start<=block_end:
            return self.config_lines[block_start:block_end+1]

    def get_block_line_numbers(self, block_type):
        config = self.config_lines
        startline = 0
        indentation_level = 0
        endline = config.__len__()-1
        block_line = False
        for line_number in range(0,self.config_lines.__len__()-1):
            logging.debug('(l=%s) (i=%s) line = %s' % (line_number, str(indentation_level), config[line_number]))
            if config[line_number] == '}':
                indentation_level-=1;
                if indentation_level == 0 and block_line:
                    logging.debug('<--- detected } for block --->')
                    endline = line_number
                    block_line = False
            if config[line_number].startswith(block_type) and indentation_level==0:
                block_line = True
                logging.debug('<--- block starts here -->')
                startline = line_number
            if config[line_number].strip().endswith('{'):
                indentation_level+=1
                if block_line and indentation_level == 1:
                    logging.debug('<--- detected { for block --->')
#                    startline = line_number+1
        if endline >= startline:
            return (startline, endline)
        return (0, 0)

    def get_devices_from_block(self, block_lines):
        indentation_level = 1
        expected_indent_level = 0
        device_line = False
        devices_list = []
        device_object = Device()
        for line in block_lines:
            logging.debug('(e=%s) , (i=%s) line = %s' % (str(expected_indent_level), str(indentation_level), line))
            if expected_indent_level == indentation_level:
                if line == '}':
                    devices_list.append(device_object)
                    device_object = Device()
                else:
                    splits = line.split(' ',1)
                    key = splits[0].strip()
                    value = splits[1].strip()
                    device_object.add_setting(key, value)
            if line[0:6]=='device' and not line[0:7] == 'devices' and indentation_level == 2: 
                device_line = True
                expected_indent_level=indentation_level+1
                device_object = Device()
            if line.endswith('{'):
                indentation_level+=1
            if line == '}':
                indentation_level-=1
        return devices_list

