#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import definitions
import time

class ConfigFileWriter(object):
    def __init__(self, filepath='/etc/multipath.conf', defaults={}, blacklisted_devnodes=[], blacklisted_devices=[], whitelisted_devices=[]):
        self.filepath = filepath
        self.defaults = defaults
        self.blacklisted_devnodes = blacklisted_devnodes
        self.blacklisted_devices = blacklisted_devices
        self.whitelisted_devices = whitelisted_devices

    def write_to_file(self):
        self.lines = []
        self.append_header()
        self.append_defaults()
        self.append_blacklist()
        self.append_whitelist()
        try:
            file_object = open(self.filepath, 'w+')
            for line in self.lines:
                file_object.write('%s\n' % line)
            file_object.flush()
            file_object.close()
        except IOError:
            logging.error('failed to open %s for writing' % self.filepath)
            return False
        return True

    def append_header(self):
        self.lines.append('# Linux MultiPath I/O Configuration File')
        self.lines.append('# Version %s' % definitions.PACKAGE_VERSION)
        (tm_year, tm_mon, tm_mday, tm_hour, tm_min, tm_sec, tm_wday, tm_yday, tm_isdst) = time.localtime()
        self.lines.append('# Generated by IBM-XIV Host Attachment at %s-%s-%s %s:%s:%s' % (tm_year, tm_mon, tm_mday, tm_hour, tm_min, tm_sec))
        self.lines.append('# Do not manually modify this file!')
        self.lines.append('')

    def append_defaults(self):
        if self.defaults.__len__():
            self.lines.append('defaults {')
            for key in self.defaults.keys():
                self.lines.append('    %s    %s' % (key, self.defaults[key]))
            self.lines.append('}')
            self.lines.append('')

    def append_blacklist(self):
        if self.blacklisted_devnodes.__len__() or self.blacklisted_devices.__len__():
            self.lines.append('blacklist {')
            for devnode in self.blacklisted_devnodes:
                self.lines.append('    devnode    %s' % (devnode))
            for device in self.blacklisted_devices:
                self.lines.append('    device {')
                for key in device.settings:
                    self.lines.append('        %s    %s' % (key, device.settings[key]))
                self.lines.append('    }')
            self.lines.append('}')
            self.lines.append('')

    def append_whitelist(self):
        if self.whitelisted_devices.__len__():
            self.lines.append('devices {')
            for device in self.whitelisted_devices:
                self.lines.append('    device {')
                for key in device.settings:
                    self.lines.append('        %s    %s' % (key, device.settings[key]))
                self.lines.append('    }')
            self.lines.append('}')
            self.lines.append('')
