#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
#
#-------------------------------------------------------------------------------------------------

import logging
import definitions
from file import File
from hostattachment.exceptions import HostAttachmentError

class UdevRules(object):
    def __init__(self):
        self.rules_source = definitions.UDEV_RULES_SOURCE
        self.udev_rules = []
        self.udev_rules.append('40-multipath.rules')
        self.udev_rules.append('45-ibm-xiv.rules')

    def is_configured(self):
        if self.dest_rules_identical_to_source_rules_dst_files() == False:
            return False
        else:
            return True

    def dest_rules_identical_to_source_rules_dst_files(self):
        for filename in self.udev_rules:
            src = '%s/%s' % (self.rules_source, filename)
            dst = File('%s/%s' % ('/etc/udev/rules.d', filename))
            try:
                diff_result = dst.identical_to_file(src)
            except IOError:
                return False
            if not diff_result:
                return False

        return True

    def configure(self):
        for filename in self.udev_rules:
            source_file = File('%s/%s' % (self.rules_source, filename))
            dest_filename  = '%s/%s' % ('/etc/udev/rules.d', filename)
            try:
                copy_result = source_file.copy_to(dest_filename)
                logging.info('udev rule %s configured successfully' % filename)
            except IOError:
                raise HostAttachmentError('failed to read udev rule file %s' %\
                    (filename) )

            if not copy_result:
                raise HostAttachmentError('failed to write udev rule file '+\
                    '%s' % (dest_filename) )

