#!/usr/bin/env python

"""
This python script detects the latest eggs of within 
the same directory as site.py pypackages 
and appends the to the path
"""
import os, sys, string

def get_list_of_eggs(directory=os.curdir):
    """ 
    returns a list of all files with an egg extension 
    from a given directory (current directory by default)
    """
    list_of_eggs = []
    for filename in os.listdir(directory):
        if filename.endswith('egg'):
            list_of_eggs.append(filename)
    return list_of_eggs

def remove_old_eggs_from_list(list_of_eggs):
    """
    given a list of egg files, this method first sorts the list, 
    then removes the older version of the same egg package
    """
    list_of_eggs.sort()
    an_old_egg = get_an_old_egg_from_list(list_of_eggs)
    while an_old_egg:
        list_of_eggs.remove(an_old_egg)
        an_old_egg = get_an_old_egg_from_list(list_of_eggs)
    
def append_list_of_eggs_to_path(list_of_eggs, eggs_base=os.curdir):
    """
    appends to the path a list of egg files by their absolute path, 
    so stuff from those eggs could get imported
    """
    for egg in list_of_eggs:
        sys.path.append(os.path.abspath(os.path.join(eggs_base,egg)))

def get_an_old_egg_from_list(list_of_eggs):
    """
    given a list of egg files, 
    this method returns the first filename of an old-version egg
    """
    for i in range(1,len(list_of_eggs)):
        if list_of_eggs[i].split('-')[0] == list_of_eggs[i-1].split('-')[0]:
            return list_of_eggs[i]
    return None
    
def main():
    """ this is the main method of site.py
    it searches for the latest egg files in the current directory, 
    and appends their absolute paths
    """
    location_of_eggs =  string.join(os.path.abspath(__file__).split(os.path.sep)[:-1],os.path.sep)
    eggs = get_list_of_eggs(location_of_eggs)
    remove_old_eggs_from_list(eggs)
    append_list_of_eggs_to_path(eggs, location_of_eggs)
    
main()

