#!/usr/bin/env xpyv

#-------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2009
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program  
# License Agreement. See the Software
# licensing materials that came with the IBM XIV Storage System for terms and
# conditions.
# 
#-------------------------------------------------------------------------------------------------   

from os import path, environ
from commands import getoutput
from subprocess import PIPE, Popen

from collector import XIVCollector
from collector.collectable.files import SimpleFileCollectable as FileItem
from collector.collectable.commands import SimpleCommandCollectable as CommandItem
from collector.collectable.directories import SimpleDirectoryCollectable as DirectoryItem

DEFAULTDIR = '%stmp' % (path.sep)
HOST_ATTACH_DIR = path.join('/', 'opt', 'xiv', 'host_attach')
HOST_ATTACH_LOGDIR = path.join(HOST_ATTACH_DIR, 'var', 'log')
HOST_ATTACH_BINDIR = path.join(HOST_ATTACH_DIR, 'bin')
HOST_ATTACH_FC = path.join(HOST_ATTACH_BINDIR, 'host_attach_fc')
HOST_ATTACH_ISCSI = path.join(HOST_ATTACH_BINDIR, 'host_attach_iscsi')

class IXIVDiag(object):
    """
    Interface describing an XIVDiag diagnostic application
    """
    def __init__(self, default_dir = DEFAULTDIR):
        self.collector = XIVCollector(default_dir)
        self.collector.get_zip_file()
        self.add_collectables()

    def __str__(self):
        return ', '.join(self.collector.collectables)
   
    def add_collectable_item(self, item):
        self.collector.add_collectable_item(item)

    def add_collectables(self):
        raise NotImplementedError()

    def __call__(self):
        self.collector.collect()
        self.collector.close()

    def set_default_collector_dir(self, directory):
        self.collector.default_dir = directory

    @property
    def zip_file(self): 
      return self.collector.zip_file
